@@ -12,7 +12,7 @@
     you may not use this file except in compliance with the License.
     You may obtain a copy of the License at
 
-        http://www.apache.org/licenses/LICENSE-2.0
+        https://www.apache.org/licenses/LICENSE-2.0
 
     Unless required by applicable law or agreed to in writing, software
     distributed under the License is distributed on an "AS IS" BASIS,
@@ -380,8 +380,8 @@ def send_email(from_addr: str,
     - https://tools.ietf.org/html/rfc2822
     - https://tools.ietf.org/html/rfc5322
     - http://segfault.in/2010/12/sending-gmail-from-python/
-    - http://stackoverflow.com/questions/64505
-    - http://stackoverflow.com/questions/3362600
+    - https://stackoverflow.com/questions/64505
+    - https://stackoverflow.com/questions/3362600
 
     Re security:
 
